# Zope
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from DateTime import DateTime
from Globals import InitializeClass

from Products.ParsedXML.ParsedXML import ParsedXML

# Silva
from Products.Silva import SilvaPermissions
from Products.Silva import IVersionedContent
from Products.Silva.VersionedContent import VersionedContent
from Products.Silva.helpers import add_and_edit,  translateCdata
#from Products.Silva import Interfaces

# custom
from helpers import check_widget


icon=None

class DisplayDescription(VersionedContent):
    """Display Description. Defines how to fill a certain slot
    """

    # XXX ok, maybe I _should_ also add some security decarations, some day
    security = ClassSecurityInfo()
    
    meta_type = "Silva Display Description"
    
    __implements__ = IVersionedContent.IVersionedContent
    

    # copy & paste from Silva/Document.py
    inherited_manage_options = VersionedContent.manage_options
    manage_options=(
        (inherited_manage_options[0],)+
        ({'label':'Silva /edit...', 'action':'edit'},)+
        inherited_manage_options[1:]
        )

    def __init__(self, id):
        DisplayDescription.inheritedAttribute('__init__')(self, id)

    
    def get_title(self):
        return self.get_container().get_title()

    def get_short_title(self):
        # XXX this has to be here, because our version object have no short title
        # which causes endless recursion in VerisonedContent.get_short_title
        return ''

    def get_view_types(self, meta_type=None):
        """ get all views for a certain meta type """
        view_registry = self.service_view_registry
        view_types=[]
        not_wanted_view_types = ('add','edit')
        for view_type in view_registry.get_view_types():
            if view_type in not_wanted_view_types:
                continue
            if not meta_type or view_registry.has_view(view_type, meta_type):
                view_types.append(view_type)
        return view_types
        
    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                             get_view_types.__name__)
        

    def to_xml(self, f):
        """Render object to XML.
        Hm, rendering the title may be superfluous ...
        """
        version = self.get_viewable()
        # cut & paste from Silva Document
        if f.last_version:
            version_id = self.get_next_version()
            if version_id is None:
                version_id = self.get_public_version()
        else:
           version_id = self.get_public_version()

        if version_id is None:
            return
        version = getattr(self, version_id)    
            
        f.write('<display_description id="%s">' % self.id)
        version.documentElement.writeStream(f)
        f.write('</display_description>')

        
InitializeClass(DisplayDescription)

manage_addDisplayDescriptionForm = PageTemplateFile("www/displayDescriptionAdd.zpt", globals(),
                                              __name__='manage_addDisplayDescriptionForm')

def manage_addDisplayDescription(self, id, REQUEST=None):
    """Add a Display Description."""
    if not self.is_id_valid(id):
        return
    object = DisplayDescription(id)
    self._setObject(id, object)
    object = getattr(self, id)
    content = '<display><index/></display>'
    # XXX using ParsedXML as version is "ancient style" ...
    object.manage_addProduct['ParsedXML'].manage_addParsedXML('0', '', content)
    object.create_version('0', None, None)
    add_and_edit(self, id, REQUEST)
    return ''


    
def registerWidgets(context):
    wr_edit = check_widget(context, 'service_display_description_editor')
    wr_edit.clearWidgets()
    
    wr_view = check_widget(context, 'service_display_description_viewer')
    wr_view.clearWidgets()
    
    #for top_item in ('display',):
    #    wr_edit.addWidget(top_item, ( 'top', top_item, 'mode_normal'))
    #    wr_view.addWidget(top_item, ( 'top', top_item, 'mode_view'))

    # reuse simple top level widget
    wr_edit.addWidget('display', ( 'service_widgets','top', 'service_display_widgets', 'top', 'display', 'mode_normal'))
    wr_view.addWidget('display', ( 'service_widgets','top', 'service_display_widgets', 'top', 'display', 'mode_view'))
    

    nested_item_names = ('index', 'a_title', 'list', 'single')

    for nested_item in nested_item_names:
        wr_edit.addWidget(nested_item,
                          ('service_widgets', 'element', 'service_display_widgets', 'element',  'display_elements', nested_item, 'mode_normal'))
        wr_view.addWidget(nested_item,
                          ('service_widgets', 'element', 'service_display_widgets', 'element', 'display_elements', nested_item, 'mode_view'))


    wr_edit.addWidget('code',
                      ( 'service_widgets', 'element', 'doc_elements', 'code', 'mode_normal'))
    wr_view.addWidget('code',
                      ( 'service_widgets', 'element', 'doc_elements', 'code', 'mode_view'))

    wr_edit.setDisplayName('display', 'Display Description')
    wr_edit.setDisplayName('index', 'Default Index')
    wr_edit.setDisplayName('a_title', 'Folder Title')
    wr_edit.setDisplayName('list', 'List of objects')
    wr_edit.setDisplayName('single', 'A certain content object')
    wr_edit.setDisplayName('code', 'Code Element')
    #wr_edit.setDisplayName('comment', 'Some more ...')

    wr_edit.setAllowed('display', nested_item_names + ('code',))


def unregisterWidgets(context):
    del_items = ['service_display_description_editor', 'service_display_description_viewer']
    del_items = [ item for item in del_items if hasattr(context.aq_base, item) ]
    context.manage_delObjects(del_items)
