# the installation for the SilvaZLBCore product

from Products.FileSystemSite.DirectoryView import manage_addDirectoryView

from Products.Silva.install import add_fss_directory_view
from Products.Silva import Document, DemoObject

import DisplayDescription

from helpers import copy_widget

def install(root):
    """install the view infrastructure for SilvaDisplayDescription.
    """
    # create the core views from filesystem
    manage_addDirectoryView(root.service_views,
                            'Products/SilvaDisplayDescription/views',
                            'SilvaDisplayDescription')
    
    registerViews(root.service_view_registry)

    # ... and the widgets:
    registerWidgets(root)

    # authors maybe want to add the content type, thus:
    configureSecurity(root)

    
def uninstall(root):
    unregisterWidgets(root)
    unregisterViews(root.service_view_registry)
    root.service_views.manage_delObjects(['SilvaDisplayDescription'])
    

def is_installed(root):
    return hasattr(root.service_views, 'SilvaDisplayDescription')

content_classes = [ DisplayDescription.DisplayDescription ]

silva_classes = [ Document.Document, DemoObject.DemoObject ]

def registerViews(reg):
    """Register views on registry.
    """

    for klass in content_classes:
        name, class_name = klass.meta_type, klass.__name__
        reg.register('public', name,
                     ['public', 'SilvaDisplayDescription', class_name])
        reg.register('add', name, 
                     [ 'add', 'SilvaDisplayDescription', class_name])
        reg.register('edit', name, 
                     [ 'edit', 'VersionedContent', 'SilvaDisplayDescription', class_name])

    for klass in silva_classes:
        name, class_name = klass.meta_type, klass.__name__    
        reg.register('summary', name,
                     ['public', 'SilvaDisplayDescription', class_name])
                      

def unregisterViews(reg):
    """Unregister views on registry.
    """
    for klass in content_classes:
        name, class_name = klass.meta_type, klass.__name__        
        for view in ('public', 'edit','add'):
            reg.unregister(view, name)

    for klass in silva_classes:
        reg.unregister('summary', klass.meta_type)

    # XXX clean "summary" view, if no other "summary" views registered
    if not reg.view_types['summary']:
        del reg.view_types['summary']


    
def registerWidgets(root):
    add_fss_directory_view(root, 'service_display_widgets', __file__, 'widgets')
    DisplayDescription.registerWidgets(root)
    copy_widget(root, root.service_doc_viewer, 'service_doc_summary_viewer')

    doc_viewer = root.service_doc_summary_viewer
    doc_viewer.addWidget('doc',
                         ('service_display_widgets','top','doc','mode_view_short'))
    
def unregisterWidgets(root):
    root.manage_delObjects(['service_doc_summary_viewer'])
    if hasattr(root,'service_display_widgets'):
        root.manage_delObjects('service_display_widgets')


def configureSecurity(root):
    """Update security permissions to only allow adding
       the content object, and its version (... not yet, as this is still a ParsedXML)
       """

    all_author = ['Author', 'Editor', 'ChiefEditor', 'Manager']
    add_permissions  = [ klass.meta_type for klass in content_classes ]
    for add_permission in add_permissions:
        root.manage_permission('Add %ss' % add_permission, all_author)    


if __name__ == '__main__':
    # positive feedback in case someone really runs it ;-)
    print """Installation successful completed."""
